      PROGRAM PreComp
!
!	This code computes structural properties of a composite blade.

!	Given blade airfoil geometry, twist distribution, composite plies layup,
!	and material properties, PreComp computes cross-sectional stiffness
!	(flap bending, lag bending, torsion, axial, coupled bending-twist,
!	axial-twist, axial-bending), mass per unit length , flap inertia,
!	lag inertia, elastic-axis offset, tension center offset, and c.m. offset)
!	at user-specified span locations.
!
!   HISTORY: -  Developed at NWTC/NREL (by Gunjit Bir), 1617, Cole Blvd., Golden, Colorado.
!	       - 6/12, RRD: modifications implemented in the treatment of the shearweb, minor bug fix
!	NOTES: 
!         1 . This code is the property of NREL.  The code, or any part of it, may
!	          not be transferred, altered or copied, directly or indirecly, without approval
!	          from NWTC/NREL.
!
!	      2. This code is still in a developmental stage and, therefore, no guarantee
!          	 is given on its accuracy and proper working. Also, the code is not fully 
!	         commented yet.
!           3. To compile use option "/real_size:64" (or "/4R8") while using ReKi = 8 (double precision)
!.................................................................................

	USE SegInfo
	USE Embed
	USE TwDist
	USE QBars
      USE Paras
	USE NWTC_Library

      REAL, ALLOCATABLE    :: chord(:)
      REAL, ALLOCATABLE    :: le_loc(:)
      REAL, ALLOCATABLE    :: ib_ch_loc(:)
      REAL, ALLOCATABLE    :: ob_ch_loc(:)
      REAL                 :: xnode(max_af_nodes) ! changed on 3-26-07
      REAL                 :: ynode(max_af_nodes) ! changed on 3-26-07
      REAL                 :: tht_lam(2,max_sectors,max_laminates) ! changed on 3-26-07
      REAL                 :: tlam(2,max_sectors,max_laminates) ! changed on 3-26-07
      REAL, ALLOCATABLE    :: tht_wlam(:,:)
      REAL, ALLOCATABLE    :: twlam(:,:)
      REAL, ALLOCATABLE    :: tphip(:)

      INTEGER                 :: n_laminas(2,max_sectors) ! changed on 3-26-07
      INTEGER                 :: mat_id (2,max_sectors,max_laminates) ! changed on 3-26-07
      INTEGER, ALLOCATABLE    :: wmat_id(:,:)
      INTEGER, ALLOCATABLE    :: n_weblams(:)
      INTEGER                 :: location(1)

      INTEGER                 :: nweb
      INTEGER                 :: idum
      INTEGER                 :: isave
      INTEGER                 :: ib_sp_stn
      INTEGER                 :: ob_sp_stn
      INTEGER                 :: n_af_nodes
      INTEGER                 :: webs_exist
      INTEGER                 :: n_materials
      INTEGER                 :: tenode_u
      INTEGER                 :: tenode_l
      INTEGER                 :: out_format

      REAL                    :: bl_length
      REAL                    :: l_web
      REAL                    :: mass
      REAL                    :: ieta1
      REAL                    :: izeta1
      REAL                    :: iepz
      REAL                    :: iemz
      REAL                    :: ipp
      REAL                    :: iqq
      REAL                    :: ipq
      REAL                    :: iflap_sc
      REAL                    :: ilag_sc
      REAL                    :: ifl_sc
      REAL                    :: iflap_cm
      REAL                    :: ilag_cm
      REAL                    :: ifl_cm
      REAL                    :: m_inertia
      REAL                    :: r_inertia
      REAL                    :: iflap_eta
      REAL                    :: ilag_zeta

      LOGICAL                 :: TabDelim

	character*99 title   ! main input file title
	character*99 af_shape_file(50), int_str_file(50) !allocate later
	character(99) :: InFile
	character(99) :: OutFile_gen
	character(99) :: OutFile_bmd
      character(99) :: fmt_data1
      character(99) :: fmt_tabs1
      character(99) :: fmt_data2
      character(99) :: fmt_tabs2

!     print version number
      call SetProg
!      write(*,*) 'This is PreComp, '//ProgVer   !RRD  testing unit 6 vs 7 for wrscr 6 works on my oc
      call DispNVD

      kr = 1
      call GET_COMMAND_ARGUMENT(1,InFile)
	call OpenFInpFile (kr, InFile)
      call NameOFile (1, 'out_gen', OutFile_gen)
      call NameOFile (1, 'out_bmd', OutFile_bmd)

!	begin reading main input file

	read (kr,*)
	read (kr,'(A)') title
	read (kr,*)
	read (kr,*)
	read (kr,*) bl_length		! actual blade length (not rotor radius)
	if(bl_length .le. 0.) then
	  write(*,*) ' ERROR** blade length not positive **'
	  stop
	endif

!	read (kr,*) bl_sca

	read (kr,*)	naf  ! number of blade secions
	if(naf .lt. 2) then
	  write(*,*) ' ERROR** number of blade stations less than two**'
	  write(*,*) ' ** specify at least two stations **'
	  stop
	endif

	read (kr,*) n_materials
!	read (kr,*) isotropy_flag
!	read (kr,*) units

	read (kr,*) out_format
	if(out_format .le. 0 .or. out_format .gt. 3) then
	  write(*,*) ' ERROR** out_format flag not recognizable **'
	  stop
	endif

      read (kr,*) TabDelim    ! output format (f: std; t: tab-delimited)

      ALLOCATE ( q11(n_materials), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate q11 in exec pgm'
       stop 8181
      endif

      ALLOCATE ( q22(n_materials), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate q22 in exec pgm'
       stop 8181
      endif

      ALLOCATE ( q12(n_materials), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate q12 in exec pgm'
       stop 8181
      endif

      ALLOCATE ( q66(n_materials), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate q66 in exec pgm'
       stop 8181
      endif

      ALLOCATE ( density(n_materials), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate density in exec pgm'
       stop 8181
      endif

	read (kr,*)
	read (kr,*)
	read (kr,*)
	read (kr,*)
	read (kr,*)
	read (kr,*)
	read (kr,*)

!	blade-sections-specific data

	call PiConsts

      ALLOCATE ( sloc(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate sloc in exec pgm'
       stop 8181
      endif

      ALLOCATE ( le_loc(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate le_loc in exec pgm'
       stop 8181
      endif

      ALLOCATE ( chord(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate chord in exec pgm'
       stop 8181
      endif

      ALLOCATE ( tw_aero(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate tw_aero in exec pgm'
       stop 8181
      endif

	do i = 1, naf

	  read (kr,*) sloc(i), le_loc(i), chord(i),
     *			  tw_aero(i), af_shape_file(i), int_str_file(i)

	  sloc(i) = sloc(i)*bl_length ! sloc dimensionalized

!	  check location of blade station
	  if (i .eq. 1) then
	   if (abs(sloc(i)) .gt. eps) then
	    write(*,*) ' ERROR** first blade station location not zero **'
	    stop
	   endif
	  else
	    if(sloc(i) .gt. bl_length) then
	     write(*,*) ' blade station location exceeds blade length **'
	     stop
	    endif
	    if(sloc(i) .lt. sloc(i-1)) then
	     write(*,*) ' blade station location decreasing **'
	     stop
	    endif
	  endif

!	  check leading edge location
	  if (le_loc(i) .lt. 0.) then
	    write(*,*) ' WARNING** leading edge aft of reference axis **'
	  endif

!	  check chord length
	  if (chord(i) .le. 0.) then
	    write(*,*) ' ERROR** chord length not positive **'
	    stop
	  endif

!	  convert twist angle to radians
	  tw_aero(i) = tw_aero(i) / r2d

	end do	! end naf loop

      ALLOCATE ( phi_prime(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate phi_prime in exec pgm'
       stop 8181
      endif

      ALLOCATE ( th_prime(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate th_prime in exec pgm'
       stop 8181
      endif

      ALLOCATE ( tphip(naf), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate tphip in exec pgm'
       stop 8181
      endif

!	get th_prime and phi_prime
	call tw_rate

	do i =1, naf
       phi_prime(i) = 0.  !later: get it from aeroelastic code
	  tphip(i) = th_prime(i) + 0.5*phi_prime(i)
	enddo

	read (kr,*)
	read (kr,*)
	read (kr,*)

	webs_exist = 1

	read (kr,*) nweb
!	  check number of webs
	  if (nweb .lt. 0) then
	    write(*,*) ' ERROR** negative number of webs **'
	    stop
	  elseif(nweb .eq. 0) then
	    write(*,*) ' ** no webs in this blade **'
		webs_exist = 0
	  endif

	if (nweb .ge. 1) then

	 read (kr,*) ib_sp_stn
!	  check inboard location of webs
	  if (ib_sp_stn .lt. 1) then
	    write(*,*) ' ERROR** web located inboard of the blade root**'
	    stop
	  endif
	  if (ib_sp_stn .gt. naf) then
	    write(*,*) ' ERROR** web inboard end past last blade stn**'
	    stop
	  endif

	 read (kr,*) ob_sp_stn
!	  check outboard location of webs
	  if (ob_sp_stn .lt. ib_sp_stn) then
	    write(*,*) ' ERROR** web outboard end location not past
     * the inboard location**'
	    stop
	  endif

	  if (ob_sp_stn .gt. naf) then
	    write(*,*) ' ERROR** web outboard end past last blade stn**'
	    stop
	  else
	    x1w = sloc(ib_sp_stn)
	    l_web = sloc(ob_sp_stn) - x1w
	  endif

!	  parameters required later for locating webs within a blade section
	  r1w = le_loc(ib_sp_stn) ! normalized
	  r2w = le_loc(ob_sp_stn) ! normalized
	  ch1 = chord(ib_sp_stn)
	  ch2 = chord(ob_sp_stn)

	 read (kr,*)
	 read (kr,*)

      ALLOCATE ( ib_ch_loc(nweb), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate ib_ch_loc in exec pgm'
       stop 8181
      endif

      ALLOCATE ( ob_ch_loc(nweb), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate ob_ch_loc in exec pgm'
       stop 8181
      endif

      ALLOCATE ( loc_web(nweb), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate loc_web in exec pgm'
       stop 8181
      endif

      ALLOCATE ( weby_u(nweb), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate weby_u in exec pgm'
       stop 8181
      endif

      ALLOCATE ( weby_l(nweb), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate weby_l in exec pgm'
       stop 8181
      endif

      ALLOCATE ( n_weblams(nweb), STAT=isttus )
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate n_weblams in exec pgm'
       stop 8181
      endif

      ALLOCATE ( tht_wlam(nweb,6), STAT=isttus ) !examine later
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate tht_wlam in exec pgm'
       stop 8181
      endif

      ALLOCATE ( wmat_id(nweb,6), STAT=isttus ) !examine later
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate wmat_id in exec pgm'
       stop 8181
      endif

      ALLOCATE ( twlam(nweb,6), STAT=isttus ) !examine later
      if(isttus /= 0) then
       write(*,*) ' ** cannot allocate twlam in exec pgm'
       stop 8181
      endif

	 do iw = 1, nweb

	  read (kr,*) idum, ib_ch_loc(iw), ob_ch_loc(iw)
!	  check web numbering
	  if (iw .eq. 1) then
	   if (idum .ne. 1) then
	    write(*,*) ' ERROR** first web must be numbered 1 **'
	    stop
	   endif
	  else
	   if(idum .ne. (isave+1)) then
	     write(*,*) ' ERROR** web numbering not sequential **'
	     stop
	   endif
	   if(ib_ch_loc(iw) .lt. ib_ch_loc(iw-1)) then
	     write(*,*) ' **ERROR webs crossing: not allowed currently **'
	   endif
	   if(ob_ch_loc(iw) .lt. ob_ch_loc(iw-1)) then
	     write(*,*) ' **ERROR webs crossing: not allowed currently **'
	   endif
	  endif
	  isave = idum

	 end do	! end nweb read loop
	endif

	write (*,*) ' main input file read successfully'

! end reading main input

!     read material properties
	open (unit = 17, file = 'materials.inp')

	read (17,*)
	read (17,*)
	read (17,*)

	do i = 1, n_materials
        read (17,*) matdum, e1, e2, g12, anu12, density(i)
	  if(anu12 .gt. sqrt(e1/e2)) then
	    write(*,921) i
921       format(1x,'**ERROR** material',i3,'properties not consistent')
	  endif
        anud = 1. - anu12*anu12*e2/e1
        q11(i) = e1/anud
        q22(i) = e2/anud
        q12(i) = anu12*e2/anud
        q66(i) = g12
	enddo

	close (unit = 17)

      if (out_format .ne. 2) then  ! general format

!        open (unit = 14, file = 'sec_props_out.gen')
        open(unit=14, file=TRIM( OutFile_gen ))

	  write (14,'(A)') '=========================================='//
     *            '========================================'
        write (14,'(A)')  'Results generated by '//TRIM( ProgName )
     & //TRIM( ProgVer )//' on '//CurDate()//' at '//CurTime()//'.'
        write(14,'(A)') title
	  write (14,'(A)') '=========================================='//
     *            '========================================'

	  write(14,691) bl_length
691     format(1x,/, t3, 'blade length (meters) =', f7.2/) 
!	  write(14,*)

        if(TabDelim ) then
           write(14, 501) (tab, itb=1,22)
501        format ('span_loc',a,'chord',a,'tw_aero',a,'ei_flap',a,
     &       'ei_lag',a,'gj',a,'ea',a,'s_fl',a,'s_af',a,'s_al',a,
     &       's_ft',a,'s_lt',a,'s_at',a,'x_sc',a,
     &       'y_sc',a,'x_tc',a,'y_tc',a,'mass',a,
     &       'flap_iner',a,'lag_iner',a,'tw_iner',a,'x_cm',a,'y_cm',/)
	  else
	     write(14,991)
991        format(1x, t3, 'span_loc', t14, 'chord', t22, 'tw_aero',
     *      t32, 'ei_flap', t45, 'ei_lag', t60, 'gj', t73, 'ea',
     *      t86, 's_fl', t99, 's_af', t112, 's_al', t124, 's_ft',
     *      t138, 's_lt', t150, 's_at', t162, 'x_sc', t171, 'y_sc',
     *      t180, 'x_tc', t189, 'y_tc',t200, 'mass', t210, 'flap_iner',
     *      t224,'lag_iner', t236, 'tw_iner',t247,'x_cm',t255,'y_cm')

	     write(14,891)
891        format(1x, t6, '(-)', t15, '(m)', t23, '(deg)',
     *      t32, '(Nm^2)', t45, '(Nm^2)', t58, '(Nm^2)', t73, '(N)',
     *      t85, '(Nm^2)', t99, '(Nm)', t112, '(Nm)', t123, '(Nm^2)',
     *      t137, '(Nm^2)', t150, '(Nm)', t163, '(m)', t172, '(m)',
     *      t181, '(m)', t190, '(m)', t199, '(Kg/m)', t212, '(Kg-m)',
     *      t225, '(Kg-m)', t237, '(deg)', t247, '(m)', t255, '(m)')
	  endif

	endif

      if (out_format .ne. 1) then   ! bmodes format
!        open (unit = 15, file = 'sec_props.bmp')
        open(unit=15, file=TRIM( OutFile_bmd ))

	  write (15,'(A)') '=========================================='//
     *            '========================================'
        write (15,'(A)')  'Results generated by '//TRIM( ProgName )
     & //TRIM( ProgVer )//' on '//CurDate()//' at '//CurTime()//'.'
        write(15,'(A)') title
	  write (15,'(A)') '=========================================='//
     *            '========================================'

	  write(15,691) bl_length

        if(TabDelim ) then
           write(15, 504) (tab, itb=1,12)
504        format('span_loc', a, 'str_tw', a, 'tw_iner',
     *      a, 'mass_den', a, 'flp_iner', a, 'edge_iner',
     *      a, 'flp_stff', a, 'edge_stff', a, 'tor_stff',
     *      a, 'axial_stff', a, 'cg_offst', a, 'sc_offst',
     *      a, 'tc_offst')
        else
	    write(15,993)
993       format(1x, t3, 'span_loc', t12, 'str_tw', t21, 'tw_iner',
     *      t31, 'mass_den', t44, 'flp_iner', t57, 'edge_iner',
     *      t70, 'flp_stff', t83, 'edge_stff', t96, 'tor_stff',
     *      t108, 'axial_stff', t120, 'cg_offst', t129, 'sc_offst',
     *      t139, 'tc_offst')

	    write(15,893)
893       format(1x, t4, '(-)', t13, '(deg)', t22, '(deg)',
     *      t32, '(kg/m)', t45, '(kg-m)', t58, '(kg-m)', t71, '(Nm^2)',
     *      t84, '(Nm^2)', t97, '(Nm^2)', t111, '(N)', t123, '(m)',
     *      t132, '(m)', t141, '(m)')
	  endif
	endif

! begin blade sections loop sec-sec-sec-sec-sec-sec-sec-sec-sec-sec--------

	do iaf = 1, naf

	write(*,901) iaf
901	format(1x, 'BLADE STATION', I4, ' analysis begins ----')
	write(*,*) '  '

	ch = chord(iaf)
      rle = le_loc(iaf)

!	read airfoil geometry
	open (unit = 12, file = af_shape_file(iaf))
	read (12,*) n_af_nodes
	read (12,*)
	read (12,*)
	read (12,*)

	if(n_af_nodes .le. 2) then
	  write(*,*)' ERROR** min 3 nodes reqd to define airfoil geometry'
	  stop
	endif

!     ALLOCATE ( xnode(n_af_nodes), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate xnode in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( ynode(n_af_nodes), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate ynode in exec pgm'
!      stop 8181
!     endif

! RRD: In lieu of allocation, resize arrays xnode and ynode to avoid problems with garbage at the end of arrays 
! The allocations above were removed, and perhaps there was a reason behind that, at this point the next two lines
! should be a good compromise between having a non-initialized allocatable array set
      xnode=xnode(1:n_af_nodes)
      ynode=ynode(1:n_af_nodes)
      
	do i = 1, n_af_nodes	   ! begin reading airfoil nodes loop
	  read (12,*) xnode(i), ynode(i)  !normalized wrt chord length
	end do					   ! end reading airfoil nodes loop

	close (unit = 12)

!	check if the first airfoil node is a leading-edge node and at (0,0)

	location = minloc(xnode)
	if(location(1) .ne. 1) then
	  write(*,*) ' ERROR** the first airfoil node not a leading node'
	  stop
	endif

	if(abs(xnode(1)) .gt. eps .or. abs(ynode(1)) .gt. eps) then
	  write(*,*) ' ERROR** leading-edge node not located at (0,0)'
	  stop
	endif

!	identify trailing-edge end nodes on upper and lower surfaces

	location = maxloc(xnode)
	if(abs(xnode(location(1))) .gt. 1.) then
	  write(*,*) ' ERROR** trailing-edge node exceeds chord boundary'
	  stop
	endif

	tenode_u = location(1)

	do i = location(1), n_af_nodes
	  if(abs(xnode(i)-xnode(location(1))).lt. eps) then
		ncounter = i - location(1)
	  endif
	end do

	tenode_l = tenode_u + ncounter

	if(ncounter .gt. 0) write(*,902) tenode_u, tenode_l
902	format(1x, 'NOTE: blunt te identified between airfoil nodes',
     *	   i4, 'and', i4)

!	renumber airfoil nodes
!	(modify later using equivalence or pointers)

	nodes_u = tenode_u
	nodes_l = n_af_nodes - tenode_l + 2

!      ALLOCATE ( xnode_u(nodes_u+50), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate xnode_u in exec pgm'
!      stop 8181
!     endif

!     ALLOCATE ( xnode_l(nodes_l+50), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate xnode_l in exec pgm'
!      stop 8181
!     endif

	do i = 1, nodes_u
	  xnode_u(i) = xnode(i)
	  ynode_u(i) = ynode(i)
	end do

	xnode_l(1) = xnode(1)
	ynode_l(1) = ynode(1)

	do i = 2, tenode_l
	  xnode_l(i) = xnode(n_af_nodes+2-i)
	  ynode_l(i) = ynode(n_af_nodes+2-i)
	end do

!	ensure surfaces are single-valued functions

	do i = 2, nodes_u
	  if((xnode_u(i) - xnode_u(i-1)) .le. eps ) then
	    write(*,*) ' ERROR** upper surface not single-valued'
		stop
	  endif
	end do

	do i = 2, nodes_l
	  if((xnode_l(i) - xnode_l(i-1)) .le. eps ) then
	    write(*,*) ' ERROR** lower surface not single-valued'
		stop
	  endif
	end do

!	check clockwise node numbering at the le

	if(ynode_u(2)/xnode_u(2) .le. ynode_l(2)/xnode_l(2)) then
	  write(*,*) ' ERROR** airfoil node numbering not clockwise'
	  stop
	endif

!	check for single-connectivity of the airfoil shape
!	(modify later using binary search)

	do j = 2, nodes_l - 1	! loop over lower-surf nodes
	  x = xnode_l(j)

	  do i = 1, nodes_u - 1	  ! loop over upper-surf nodes

		 xl = xnode_u(i)
		 xr = xnode_u(i+1)

		 if(x .ge. xl .and. x .le. xr) then
		   yl = ynode_u(i)
		   yr = ynode_u(i+1)
		   y = yl + (yr-yl)*(x-xl)/(xr-xl)
	       if(ynode_l(j) .ge. y) then
		     write(*,*) ' ERROR** airfoil shape self-crossing'
			  stop
		    endif
	     endif

	  enddo	   ! end loop over upper-surf nodes

	end do   ! end loop over lower-surf nodes


!	find chordwise location of webs from the l.e. (current blade section)

	if (nweb .ge. 1) then  ! if webs exist ------

	 webs_exist = 1
       if(iaf .lt. ib_sp_stn .or. iaf .gt. ob_sp_stn) then

	  write(*,*) ' No web at this blade station'
	  webs_exist = 0
	 else

	  xlocn = (sloc(iaf)-x1w)/l_web

	  do i = 1, nweb
	    p1w = ib_ch_loc(i)
	    p2w = ob_ch_loc(i)
		 loc_web(i) = rle - (r1w-p1w)*ch1*(1.-xlocn)/ch
     *				-(r2w-p2w)*ch2*xlocn/ch		!chord-normalized web location
	    if(loc_web(i) .lt. 0. .or. loc_web(i) .gt. 1.) then
	      write(*,903) i
903	      format(1x,'ERROR** web no', i2, ' outside airfoil boundary',
     *      ' at the current blade station')
	      stop
	    endif
	  end do

	 endif

	endif  ! end webs location computation  ----------

!	embed airfoil nodes at web-to-airfoil intersections

	if (webs_exist .eq. 1) then
	 do i = 1, nweb
	  call embed_us(loc_web(i), ynd)
	  weby_u(i) = ynd
	  call embed_ls(loc_web(i), ynd)
	  weby_l(i) = ynd
	 end do
	endif

!!	read internal srtucture data
	open (unit = 13, file = int_str_file(iaf))

!     ALLOCATE ( xsec_node(2,11), STAT=isttus ) !examine later
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate xsec_node in exec pgm'
!      stop 8181
!     endif

!     ALLOCATE ( n_laminas(2,10), STAT=isttus ) !examine later
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate n_laminas in exec pgm'
!      stop 8181
!     endif

!     ALLOCATE ( tht_lam(2,10,10), STAT=isttus ) !examine later
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate tht_lam in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( mat_id(2,10,10), STAT=isttus ) !examine later
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate mat_id in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( tlam(2,10,10), STAT=isttus ) !examine later
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate tlam in exec pgm'
!      stop 8181
!     endif

	do is = 1, 2  ! begin loop for blade surfaces

		read (13,*)
		read (13,*)
		read (13,*)
		read (13,*) n_scts(is)
		nsects = n_scts(is)

		if (nsects .le. 0) then
		  write(*,*) ' ERROR** no of sectors not positive'
		  stop
		endif

		read (13,*)
		read (13,*)

	   read (13,*) (xsec_node(is,isect), isect = 1, nsects+1)

		if (xsec_node(is,1) .lt. 0.) then
		  write(*,*) ' ERROR** sector node x-location not positive'
		endif

		if (is .eq. 1) then
	      xu1 = xsec_node(is,1)
		  xu2 = xsec_node(is,nsects+1)
		  if (xu2 .gt. xnode_u(nodes_u)) then
		    write(*,*)
     *      ' ERROR** upper-surf last sector node out of bounds'
		    stop
		  endif
		else
	      xl1 = xsec_node(is,1)
		  xl2 = xsec_node(is,nsects+1)
		  if (xl2 .gt. xnode_l(nodes_l)) then
		    write(*,*)
     *      ' ERROR** lower-surf last sector node out of bounds'
		    stop
		  endif
		endif

		do i = 1, nsects
		  if (xsec_node(is,i+1) .le. xsec_node(is,i)) then
		    write(*,*)
     *	  ' ERROR** sector nodal x-locations not in ascending order'
		    stop
		  endif
		end do

		do isect = 1, nsects  !begin sectors loop

		  read (13,*)
		  read (13,*)
		  read (13,*) idum, n_laminas(is,isect)

		  if (idum .ne. isect) then
			write(*,904) idum
904			format(1x, 'ERROR**', i2,
     *		'is a wrong or out-of-sequence sector number')
			stop
		  endif

		  read (13,*)
		  read (13,*)
		  read (13,*)
		  read (13,*)

		  do lam = 1, n_laminas(is,isect)  !begin laminae loop

		    read (13,*) idum, n_plies, tply, tht_lam(is,isect,lam),
     *		 mat_id(is,isect,lam)

		    if (idum .ne. lam) then
		      write(*,906) idum
906		      format(1x, 'ERROR**', i2,
     *	     'is a wrong or out-of-sequence lamina number')
		      stop
	        endif

			tlam(is,isect,lam) = n_plies*tply
			tht_lam(is,isect,lam) = tht_lam(is,isect,lam)/r2d
		  end do  !end laminae loop

		end do	! end sectors loop

!		embed airfoil nodes representing sectors bounds

	    do i = 1, nsects+1

		  if(is .eq. 1) then

		    call embed_us(xsec_node(is,i), ynd)

			if(i .eq. 1) then
			  yu1 = ynd
			  ndu1 = newnode
			endif
			if(i .eq. nsects+1) then
			  yu2 = ynd
			  ndu2 = newnode
			endif

		  endif

		  if(is .eq. 2) then

		    call embed_ls(xsec_node(is,i), ynd)

			if(i .eq. 1) then
			  yl1 = ynd
			  ndl1 = newnode
			endif
			if(i .eq. nsects+1) then
			  yl2 = ynd
			  ndl2 = newnode
			endif

		  endif

		enddo

	end do		! end blade surfaces loop


!.... check for le and te non-closures and issue warning ....

	if (abs(xu1-xl1) .gt. eps) then

	  write(*,*)
     *' WARNING** the leading edge may be open; check closure'

	else

	  if ((yu1-yl1) .gt. eps) then
		wreq = 1

	    if (webs_exist .ne. 0) then
	        if (abs(xu1-loc_web(1)) .lt. eps) then
	          wreq = 0
			endif
		endif

	    if (wreq .eq. 1) write(*,*)
     *	    ' WARNING** open leading edge; check web requirement'

	  endif

	endif

!
	if (abs(xu2-xl2) .gt. eps) then

	  write(*,*)
     *' WARNING** the trailing edge may be open; check closure'

	else

	  if ((yu2-yl2) .gt. eps) then
	  wreq = 1

	    if (webs_exist .ne. 0) then
	        if (abs(xu2-loc_web(nweb)) .lt. eps) then
	          wreq = 0
			endif
		endif

	    if (wreq .eq. 1) write(*,*)
     *	    ' WARNING** open trailing edge; check web requirement'

	  endif

	endif
!................

	if (webs_exist .eq. 1) then

		read (13,*)
		read (13,*)
		read (13,*)
		read (13,*)

	  do iweb = 1, nweb
		read (13,*)
		read (13,*)
		read (13,*) idum, n_weblams(iweb)

		if (idum .ne. iweb) then
		  write(*,907) idum
907		  format(1x, 'ERROR**', i2,
     *	  'is a wrong or out-of-sequence web number')
		  stop
		endif

		read (13,*)
		read (13,*)
		read (13,*)
		read (13,*)

		do lam = 1, n_weblams(iweb)
		  read (13,*) idum, n_plies, tply, tht_wlam(iweb,lam),
     *				  wmat_id(iweb,lam)

		  if (idum .ne. lam) then
		      write(*,908) idum
908		      format(1x, 'ERROR**', i2,
     *	     'is a wrong or out-of-sequence web lamina number')
		      stop
	      endif

		  twlam(iweb,lam) = n_plies*tply
		  tht_wlam(iweb,lam) = tht_wlam(iweb,lam)/r2d
		enddo

	  enddo

	  if(loc_web(1) .lt. xu1 .or. loc_web(1) .lt. xl1) then
	    write(*,*) ' ERROR** first web out of sectors-bounded airfoil'
	    stop
	  endif

	  if(loc_web(nweb) .gt. xu2 .or. loc_web(nweb) .gt. xl2) then
	    write(*,*) ' ERROR** last web out of sectors-bounded airfoil'
	    stop
	  endif

	endif

	close (unit = 13)

	write(*,*) ' all inputs successfully read and checked'
!!

!	identify segments groupings and get segs attributes
	nseg_u = ndu2 - ndu1
	nseg_l = ndl2 - ndl1
	nseg_p = nseg_u + nseg_l	! no of peripheral segments

	if(webs_exist .eq. 1) then
       nseg = nseg_p + nweb  ! total no of segments (webs in section)
      else
       nseg = nseg_p      ! total no of segments (no webs in section)
      endif

	call seg_info
!------------------------------------------

!	initialize for section (sc)
	sigma = 0.
	eabar = 0.
	q11ya = 0.
	q11za = 0.

!	segments loop for sc

!     ALLOCATE ( isur(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate isur in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( idsect(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate idsect in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( yseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate yseg in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( zseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate zseg in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( wseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate wseg in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( sthseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate sthseg in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( cthseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate cthseg in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( s2thseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate s2thseg in exec pgm'
!      stop 8181
!     endif
!
!     ALLOCATE ( c2thseg(nseg), STAT=isttus )
!     if(isttus /= 0) then
!      write(*,*) ' ** cannot allocate c2thseg in exec pgm'
!      stop 8181
!     endif

	do iseg = 1, nseg_p	!begin paeripheral segments loop (sc)

!	  retrieve seg attributes
	  is = isur(iseg)
	  idsec = idsect(iseg)
	  ysg = yseg(iseg)
	  zsg = zseg(iseg)
	  w = wseg(iseg)
	  sths = sthseg(iseg)
	  cths = cthseg(iseg)
!	  s2ths = s2thseg(iseg)
!	  c2ths = c2thseg(iseg)  ! commented out

	  nlam = n_laminas(is,idsec)	! for sector seg

!	  initialization for seg (sc)
	  tbar = 0.
	  q11t = 0.
	  q11yt_u = 0.
	  q11zt_u = 0.
	  q11yt_l = 0.
	  q11zt_l = 0.

	  do ilam = 1, nlam	!laminas loop (sc)

		  t = tlam(is,idsec,ilam)		   !thickness
		  thp = tht_lam(is,idsec,ilam)	! ply angle
		  mat = mat_id(is,idsec,ilam)    !material

		  tbar = tbar + t/2.
		  y0 = ysg - ((-1)**(is))*tbar*sths
		  z0 = zsg + ((-1)**(is))*tbar*cths

!		obtain qtil for specified mat
		call q_bars
		call q_tildas

!		add seg-laminas contributions (sc)
		qtil11t = qtil(1,1)*t
		q11t = q11t + qtil11t
		if(iseg .le. nseg_u) then
           q11yt_u = q11yt_u + qtil11t*y0
		  q11zt_u = q11zt_u + qtil11t*z0
	   else
           q11yt_l = q11yt_l + qtil11t*y0
		  q11zt_l = q11zt_l + qtil11t*z0
	   endif
!      if (iaf .eq. 1) then
!        write(33,*) iseg, ilam, qtil11t, y0, z0 !gbxx
!	endif
		tbar = tbar + t/2.

	  enddo			! end laminas loop

!		add seg contributions (sc)
		sigma = sigma + w*abs(zsg + ((-1)**is)*0.5*tbar*cths)*cths
		eabar = eabar + q11t*w
		q11ya = q11ya + (q11yt_u+q11yt_l)*w
		q11za = q11za + (q11zt_u+q11zt_l)*w

	enddo			!end af_periph segment loop (sc)

!	 get section sc
	 y_sc =	q11ya/eabar		!wrt r-frame
	 z_sc =	q11za/eabar		!wrt r-frame

!---------------- end section sc -----------


!	initializations for section (properties)

	eabar = 0.
	q11ya = 0.
	q11za = 0.
	ap = 0.
	bp = 0.
	cp = 0.
	dp = 0.
	ep = 0.
	q11ysqa = 0.
	q11zsqa = 0.
	q11yza = 0.

	mass = 0.
	rhoya = 0.
	rhoza = 0.
	rhoysqa = 0.
	rhozsqa = 0.
	rhoyza = 0.

!	segments loop (for properties)

	do iseg = 1, nseg	!begin segment loop (properties)

!	  retrieve seg attributes
	  is = isur(iseg)
	  idsec = idsect(iseg)
	  ysg = yseg(iseg)
	  zsg = zseg(iseg)
	  w = wseg(iseg)
	  sths = sthseg(iseg)
	  cths = cthseg(iseg)
	  s2ths = s2thseg(iseg)
	  c2ths = c2thseg(iseg)

	  if(is .gt. 0) then
		nlam = n_laminas(is,idsec)	! for sector seg
	  else
		iweb = idsec
		nlam = n_weblams(iweb)		! for web seg
	  endif

!	  initialization for seg (properties)
	  tbar = 0.
	  q11t = 0.
	  q11yt = 0.
	  q11zt = 0.
	  dtbar = 0.
	  q2bar = 0.
	  zbart = 0.
	  ybart = 0.
	  tbart = 0.
	  q11ysqt = 0.
	  q11zsqt = 0.
	  q11yzt = 0.

	  rhot = 0.
	  rhoyt = 0.
	  rhozt = 0.
	  rhoysqt = 0.
	  rhozsqt = 0.
	  rhoyzt = 0.

	  do ilam = 1, nlam	!laminas loop (properties)

		if(is .gt. 0) then
		  t = tlam(is,idsec,ilam)		   !thickness
		  thp = tht_lam(is,idsec,ilam)	! ply angle
		  mat = mat_id(is,idsec,ilam)	   ! material
		  tbar = tbar + t/2.
		  y0 = ysg - ((-1)**(is))*tbar*sths - y_sc
		  z0 = zsg + ((-1)**(is))*tbar*cths - z_sc
		else
		  t = twlam(iweb,ilam)
		  thp = tht_wlam(iweb,ilam)
		  mat = wmat_id(iweb,ilam)
		  tbar = tbar + t/2.
		  y0 = ysg - tbar/2. - y_sc !RRD: this had y0 in place of ysg; fixed.
		  z0 = zsg - z_sc
		endif

	      y0sq = y0*y0
	      z0sq = z0*z0

!		obtain qtil and rho for specified mat
		call q_bars
		call q_tildas


		 ieta1 = (t**2)/12.0
		 izeta1 = (w**2)/12.0
		 iepz = 0.5*(ieta1+izeta1)
		 iemz = 0.5*(ieta1-izeta1)
		 ipp = iepz + iemz*c2ths   ! check this block later !RRD: changed -to+
		 iqq = iepz - iemz*c2ths                            !RRD: changed +to-
		 ipq = iemz*s2ths

		 qtil11t = qtil(1,1)*t
		 rot = rho_m*t

!		add laminas contributions (properties) at current segment

		if(is .gt. 0) then ! peripheral segs contribution

		 qtil12t = qtil(1,2)*t
		 qtil22t = qtil(2,2)*t

		 q11t = q11t + qtil11t
		 q11yt = q11yt + qtil11t*y0
		 q11zt = q11zt + qtil11t*z0

		 dtbar = dtbar + qtil12t*(y0sq+z0sq)*tphip(iaf)*t
		 q2bar = q2bar + qtil22t    ! later: retain only this block
		 zbart = zbart + z0*qtil12t
		 ybart = ybart + y0*qtil12t
		 tbart = tbart + qtil12t

		 q11ysqt = q11ysqt + qtil11t*(y0sq+iqq)
	 	 q11zsqt = q11zsqt + qtil11t*(z0sq+ipp)
		 q11yzt = q11yzt + qtil11t*(y0*z0+ipq)

		 rhot = rhot + rot
		 rhoyt = rhoyt + rot*y0
		 rhozt = rhozt + rot*z0
		 rhoysqt = rhoysqt + rot*(y0sq+iqq)
	 	 rhozsqt = rhozsqt + rot*(z0sq+ipp)
		 rhoyzt = rhoyzt + rot*(y0*z0+ipq)

		else			!web segs contribution

		 q11t = q11t + qtil11t
		 q11yt = q11yt + qtil11t*y0
		 q11zt = q11zt + qtil11t*z0
		 q11ysqt = q11ysqt + qtil11t*(y0sq+iqq)
	 	 q11zsqt = q11zsqt + qtil11t*(z0sq+ipp)
		 q11yzt = q11yzt + qtil11t*(y0*z0+ipq)

		 rhot = rhot + rot
		 rhoyt = rhoyt + rot*y0
		 rhozt = rhozt + rot*z0
		 rhoysqt = rhoysqt + rot*(y0sq+iqq)
	 	 rhozsqt = rhozsqt + rot*(z0sq+ipp)
		 rhoyzt = rhoyzt + rot*(y0*z0+ipq)

		endif

		tbar = tbar + t/2.

	  enddo			! end laminas loop

!		add seg contributions to obtain sec props about ref_parallel axes at sc
		eabar = eabar + q11t*w
		q11ya = q11ya + q11yt*w
		q11za = q11za + q11zt*w
		q11ysqa = q11ysqa + q11ysqt*w
		q11zsqa = q11zsqa + q11zsqt*w
		q11yza = q11yza + q11yzt*w

		if(is .gt. 0) then
           wdq2bar = w/q2bar
		  ap = ap + wdq2bar
		  bp = bp + wdq2bar*tbart
		  cp = cp + wdq2bar*dtbar
		  dp = dp + wdq2bar*zbart
		  ep = ep + wdq2bar*ybart
	   endif

		mass = mass + rhot*w
		rhoya = rhoya + rhoyt*w
		rhoza = rhoza + rhozt*w
		rhoysqa = rhoysqa + rhoysqt*w
		rhozsqa = rhozsqa + rhozsqt*w
		rhoyza = rhoyza + rhoyzt*w


	enddo		!end af_periph segment loop (properties)

!	 get more section properties ! about ref_parallel axes at sc

	 y_tc =	q11ya/eabar
	 z_tc =	q11za/eabar

	 sfbar = q11za
	 slbar = q11ya
	 eifbar = q11zsqa
	 eilbar = q11ysqa
	 eiflbar = q11yza

	 sigm2 = sigma*2.
	 gjbar = sigm2*(sigm2+cp)/ap
	 sftbar = -sigm2*dp/ap
	 sltbar = -sigm2*ep/ap
	 satbar = sigm2*bp/ap

	 ycm_sc =	rhoya/mass !wrt sc
	 zcm_sc =	rhoza/mass !wrt sc

	 iflap_sc = rhozsqa !wrt sc
	 ilag_sc = rhoysqa	 !wrt sc
	 ifl_sc = rhoyza	 !wrt sc

!	 get section tc and cm

	 ytc_ref =	y_tc + y_sc	 !wrt the ref axes
	 ztc_ref =	z_tc + z_sc	 !wrt the ref axes

	 ycm_ref =	ycm_sc + y_sc	 !wrt the ref axes
	 zcm_ref =	zcm_sc + z_sc	 !wrt the ref axes

!	 moments of inertia ! about ref_parallel axes at cm

	 iflap_cm = iflap_sc - mass*zcm_sc**2
	 ilag_cm = ilag_sc - mass*ycm_sc**2
	 ifl_cm = ifl_sc - mass*ycm_sc*zcm_sc

!	 inertia principal axes orientation and moments of inertia

	 m_inertia = 0.5*(ilag_cm + iflap_cm)
	 r_inertia = sqrt(0.25*((ilag_cm-iflap_cm)**2) + ifl_cm**2)

	 if(iflap_cm .le. ilag_cm) then
	   iflap_eta = m_inertia - r_inertia
	   ilag_zeta = m_inertia + r_inertia
	 else
	   iflap_eta = m_inertia + r_inertia
	   ilag_zeta = m_inertia - r_inertia
	 endif

	 if(ilag_cm .eq. iflap_cm) then
	   th_pa = pi/4.
	   if(abs(ifl_cm/iflap_cm) .lt. 1.e-6) th_pa = 0.
	 else
	   th_pa = 0.5*abs(atan(2.*ifl_cm/(ilag_cm-iflap_cm)))
	 endif

	 if(abs(ifl_cm) .lt. eps) then
	  th_pa = 0.
	 else          ! check this block later
	  if(iflap_cm .ge. ilag_cm .and. ifl_cm .gt. 0.) th_pa = -th_pa
	  if(iflap_cm .ge. ilag_cm .and. ifl_cm .lt. 0.) th_pa = th_pa
	  if(iflap_cm .lt. ilag_cm .and. ifl_cm .gt. 0.) th_pa = th_pa
	  if(iflap_cm .lt. ilag_cm .and. ifl_cm .lt. 0.) th_pa = -th_pa
 	 endif

!	 elastic principal axes orientation and principal bending stiffneses

	 em_stiff = 0.5*(eilbar + eifbar)
	 er_stiff = sqrt(0.25*((eilbar-eifbar)**2) + eiflbar**2)

	 if(eifbar .le. eilbar) then
	   pflap_stff = em_stiff - er_stiff
	   plag_stff = em_stiff + er_stiff
	 else
	   pflap_stff = em_stiff + er_stiff
	   plag_stff = em_stiff - er_stiff
	 endif

	 if(eilbar .eq. eifbar) then
	   the_pa = pi/4.
	 else
	   the_pa = 0.5*abs(atan(2.*eiflbar/(eilbar-eifbar)))
	 endif

	 if(abs(eiflbar) .lt. eps) then
	  the_pa = 0.
	 else          ! check this block later
	  if(eifbar .ge. eilbar .and. eiflbar .gt. 0.) the_pa = -the_pa
	  if(eifbar .ge. eilbar .and. eiflbar .lt. 0.) the_pa = the_pa
	  if(eifbar .lt. eilbar .and. eiflbar .gt. 0.) the_pa = the_pa
	  if(eifbar .lt. eilbar .and. eiflbar .lt. 0.) the_pa = -the_pa
 	 endif

!---------------- end properties computation -----------

!     output section properties

      id_form = 1  ! hardwired for wt's

      if (id_form .eq. 1) then
        tw_iner = tw_aero(iaf) - th_pa
	  str_tw =  tw_aero(iaf) - the_pa
        y_sc = -y_sc
        ytc_ref = -ytc_ref
        ycm_ref = -ycm_ref
	else         ! for h/c
!       note: for h/c, th_aero input is +ve acc to h/c convention
	  tw_iner = tw_aero(iaf) + th_pa
	  str_tw =  tw_aero(iaf) + the_pa
	endif

      if (out_format .ne. 2) then  ! general format

        if(TabDelim ) then
	    write(14,503) sloc(iaf)/bl_length,tab,chord(iaf),tab, 
     *      tw_aero(iaf)*r2d,tab,eifbar,tab,eilbar,tab,gjbar,tab,eabar,
     *      tab, -eiflbar,tab, -sfbar,tab,slbar,tab,sftbar,tab,-sltbar,
     *      tab,satbar,tab,z_sc,tab,y_sc,tab,ztc_ref,tab,ytc_ref,tab,
     *     mass,tab,iflap_eta,tab,ilag_zeta,tab,tw_iner*r2d,tab,
     *     zcm_ref,tab,ycm_ref
503       format(f8.3, a, f8.3, a, f8.3, 10(a, e12.4),
     *		 4(a, f8.3), 3(a, e12.4), 3(a, f8.3))
	  else
	    write(14,992) sloc(iaf)/bl_length, chord(iaf), tw_aero(iaf)*r2d, 
     *      eifbar, eilbar, gjbar, eabar, -eiflbar, -sfbar, slbar,
     *      sftbar, -sltbar, satbar, z_sc, y_sc, ztc_ref, ytc_ref,
     *      mass, iflap_eta, ilag_zeta, tw_iner*r2d, zcm_ref, ycm_ref
992       format(1x, f8.3, 1x, f8.3, 1x, f8.3, 10(1x, e12.4),
     *		 4(1x, f8.3), 3(1x, e12.4), 3(1x, f8.3))
	  endif

	endif

      if (out_format .ne. 1) then  ! bmodes format

	  if(TabDelim ) then
	    write(15,506) sloc(iaf)/bl_length,tab,str_tw*r2d,tab,
     *       tw_iner*r2d,tab,mass,tab,iflap_eta,tab,ilag_zeta,tab,
     *       pflap_stff,tab,plag_stff,tab,gjbar,tab,eabar,tab,ycm_ref,
     *       tab,y_sc,tab,ytc_ref
506       format(f7.4,a,f8.3,a,f8.3,7(a,e12.4), 3(a,f8.3))
	  else
	    write(15,995) sloc(iaf)/bl_length, str_tw*r2d, 
     *       tw_iner*r2d, mass, iflap_eta, ilag_zeta, pflap_stff,
     *       plag_stff, gjbar, eabar, ycm_ref, y_sc, ytc_ref
995       format(1x,f7.4,1x,f8.3,1x,f8.3,7(1x,e12.4), 3(1x,f8.3))
        endif

	endif

!
	end do			! end blade sections loop -------------------

!     later: compute total beam mass and its c.m.

      call normstop
!      stop
	end
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	subroutine embed_us(x,y)

!	purpose: embed a node in the upper-surface airfoil section nodes

	USE Embed
      USE Paras

	real x, y

!	NOTE: nodal x coordinates must be in ascending order

!	inputs:
!	  x					: x-coordinate of node to be embedded in the u-surf
!	  nodes_u			: no of current nodes on the upper surface
!	  xnode_u(nodes_u)	: x-coord of u-surf nodes
!	  ynode_u(nodes_u)	: y-coord of u-surf nodes

!	outputs:
!	  y					: y-coordinate of node embedded in the u-surf
!!	  nodes_u			: revised no of current nodes on the upper surface
!	  xnode_u(nodes_u)	: x-coord of revised u-surf nodes
!	  ynode_u(nodes_u)	: y-coord of revised u-surf nodes
!	  newnode			: number of the embedded node


	newnode = -1
!!
	do i = 1, nodes_u - 1	! loop over upper-surf nodes
	  xl = xnode_u(i)
	  xr = xnode_u(i+1)
	  yl = ynode_u(i)

	  if(abs(x-xl) .le. eps) then
	    newnode = 0
		isave = i
		y = yl
		exit
	  else
		if(x .lt. (xr-eps)) then
		  yr = ynode_u(i+1)
		  y = yl + (yr-yl)*(x-xl)/(xr-xl)
	      newnode = i+1
		  exit
		endif
	  endif
	enddo		! end loop over upper-surf nodes
!!
	if(newnode .eq. -1) then
	  if(abs(x-xnode_u(nodes_u)) .le. eps) then
	    newnode = 0
		isave = nodes_u
		y =  ynode_u(nodes_u)
	  else
	    write(*,*) ' ERROR unknown, consult NWTC'
	    stop 1111
	  endif
	endif
!!
	if(newnode .gt. 0) then
	  nodes_u = nodes_u +1

	  do i = nodes_u, (newnode+1), -1
		xnode_u(i) = xnode_u(i-1)
		ynode_u(i) = ynode_u(i-1)
	  enddo
!!
	  xnode_u(newnode) = x
	  ynode_u(newnode) = y

	else
	  newnode = isave
	endif

	return
	end

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


	subroutine embed_ls(x,y)

!	purpose: embed a node in the lower-surface airfoil section nodes

	USE Embed
      USE Paras

	real x, y

!	NOTE: nodal x coordinates must be in ascending order

!	inputs:
!	  x					: x-coordinate of node to be embedded in the l-surf
!	  nodes_l			: no of current nodes on the lower surface
!	  xnode_l(nodes_l)	: x-coord of l-surf nodes
!	  ynode_l(nodes_l)	: y-coord of l-surf nodes

!	outputs:
!	  y					: y-coordinate of node embedded in the l-surf
!!	  nodes_l			: revised no of current nodes on the lower surface
!	  xnode_l(nodes_l)	: x-coord of revised l-surf nodes
!	  ynode_l(nodes_l)	: y-coord of revised l-surf nodes
!	  newnode			: number of the embedded node


	newnode = -1
!!
	do i = 1, nodes_l - 1	! loop over lower-surf nodes
	  xl = xnode_l(i)
	  xr = xnode_l(i+1)
	  yl = ynode_l(i)

	  if(abs(x-xl) .le. eps) then
	    newnode = 0
		isave = i
		y = yl
		exit
	  else
		if(x .lt. (xr-eps)) then
		  yr = ynode_l(i+1)
		  y = yl + (yr-yl)*(x-xl)/(xr-xl)
	      newnode = i+1
		  exit
		endif
	  endif

	enddo		! end loop over lower-surf nodes
!!
	if(newnode .eq. -1) then
	  if(abs(x-xnode_l(nodes_l)) .le. eps) then
	    newnode = 0
		isave = nodes_l
		y =  ynode_l(nodes_l)
	  else
	    write(*,*) ' ERROR unknown, consult NWTC'
	    stop 1111
	  endif
	endif
!!
	if(newnode .gt. 0) then
	  nodes_l = nodes_l +1

	  do i = nodes_l, (newnode+1), -1
		xnode_l(i) = xnode_l(i-1)
		ynode_l(i) = ynode_l(i-1)
	  enddo
!!
	  xnode_l(newnode) = x
	  ynode_l(newnode) = y

	else
	  newnode = isave
	endif

	return
	end

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	subroutine seg_info

	USE SegInfo
	USE Embed
      USE Paras
	USE NWTC_Library

!	NOTE: coord transformation from xaf-yaf to yre-zref and seg info

!	inputs:
!	  ch			: chord length
!	  eps			: epsilon parameter  MACHINE PRECISION
!	  nseg			: total number of segs
!	  nseg_u		: no of segs on the upper surface
!	  nseg_p		: no of segs for both upper and lower surfaces
!	  xnode_u(i)	: x coord of node i at the upper surf
!	  ynode_u(i)	: y coord of node i at the upper surf
!	  xnode_l(i)	: x coord of node i at the lower surf
!	  ynode_l(i)	: y coord of node i at the lower surf
!	  ndl1			: 1st seg lhs node number (lower surf)
!	  ndu1			: 1st seg lhs node number (upper surf)
!	  rle			: loc of l.e. (non-d wrt chord)
!	  loc_web(iweb)	: x coord of web no iweb
!	  weby_u(iweb)	: y coord of web no iweb on the upper surf
!	  weby_l(iweb)	: y coord of web no iweb on the lower surf
!	  n_scts(is)	: no of sectors on 'is' surf
!	  xsec_node(s,i): x coord of sect-i lhs on 's' surf
!
!	outputs:
!	  isur(iseg)	: surf id
!	  idsect(iseg)	: associated sect or web number
!	  yseg(iseg)	: y-ref of mid-seg point
!	  zseg(iseg)	: z-ref of mid-seg point
!	  wseg(iseg)	: seg width
!	  sthseg(iseg)	: sin(th_seg)
!	  cthseg(iseg)	: cos(th_seg)
!	  s2thseg(iseg)	: sin(2*th_seg)
!	  c2thseg(iseg) : cos(2*th_seg)


       do iseg = 1, nseg   ! seg numbering from le clockwise
	  is = -1
	 if(iseg .le. nseg_u) then	! upper surface segs

		 nd_a = ndu1+iseg-1
		 xa = xnode_u(nd_a)
		 ya = ynode_u(nd_a)
		 xb = xnode_u(nd_a+1)
		 yb = ynode_u(nd_a+1)
		 is = 1
	 else
		if(iseg .le. nseg_p) then	! lower surface segs
		  nd_a = ndl1+iseg-nseg_u-1
		  xa = xnode_l(nd_a)		!xref of node toward le (in a/f ref frame)
		  ya = ynode_l(nd_a)		!yref of node toward le (in new ref frame)
		  xb = xnode_l(nd_a+1)		!xref of node toward te (in a/f ref frame)
		  yb = ynode_l(nd_a+1)		!yref of node toward te (in new ref frame)
		  is = 2
		endif

		if(iseg .gt. nseg_p ) then	! web segs
		  iweb = iseg-nseg_p
		  xa = loc_web(iweb)
		  xb = xa
		  ya = weby_u(iweb)
		  yb = weby_l(iweb)
		  is = 0
		endif

	 endif	! end seg group identification

	 if(is .eq. -1) then
         write(*,*) 'iseg=', iseg
		write(*,*) ' ERROR** unknown, contact NREL'
		stop 3333
	 endif

	 isur(iseg) = is

	 if(is .gt. 0) then	!id assocaited sect number
	  icheck = 0
	  do i = 1, n_scts(is)
		if ( (xa .gt. (xsec_node(is,i)-eps)) .and.
     *	  (xb .lt. (xsec_node(is,i+1)+eps)) ) then
		  idsect(iseg) = i
	      icheck = 1
		  exit
		endif
	  enddo
	 endif

	 if(icheck .eq. 0) then
           write(*,*) ' ERROR** unknown, contact NREL'
		  stop 2222
	 endif

	 if(is .eq. 0) idsect(iseg) = iweb	 !id associated web number

	 xba = xb - xa
	 yba = ya - yb
	 yseg(iseg) = ch*(2.*rle-xa-xb)/2. !yref coord of mid-seg pt (in r-frame)
	 zseg(iseg) = ch*(ya+yb)/2.	   !zref coord of mid-seg pt (in r-frame)
	 wseg(iseg) = ch*sqrt(xba**2 + yba**2)

	 if (is .eq. 0) then
	  thseg = -pi/2.
	 else
	  thseg = atan(yba/xba)	! thseg +ve in new y-z ref frame
	 endif

	 sthseg(iseg) = sin(thseg)
	 cthseg(iseg) = cos(thseg)
	 s2thseg(iseg) = sin(2*thseg)
	 c2thseg(iseg) = cos(2*thseg)

	enddo	! end seg loop

	return
	end

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	subroutine tw_rate

	USE TwDist

!	inputs:
!	 naf: no of blade stations
!	 sloc(naf): vector of station locations
!	 tw_aero(naf): vector of twist distribution

!	outputs:
!	 th_prime(naf): vector of twist rates

	do i = 2, naf-1
	  f0 = tw_aero(i)
	  f1 = tw_aero(i-1)
	  f2 = tw_aero(i+1)
	  h1 = sloc(i) - sloc(i-1)
	  h2 = sloc(i+1) - sloc(i)
	  th_prime(i) = (h1*(f2-f0) + h2*(f0-f1))/(2.*h1*h2)
	enddo

	th_prime(1) = (tw_aero(2)-tw_aero(1))/(sloc(2)-sloc(1))
	th_prime(naf)=(tw_aero(naf)-tw_aero(naf-1))/(sloc(naf)-sloc(naf-1))

	return
	end

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	subroutine q_bars

	USE QBars

	real k11, k22, k12, k66, kmm, kmp

!     inputs:
!     mat: material id
!     thp: ply orientation
!     density(n_materials): material densities
!     q11(n_materials), q22(n_materials), q12(n_materials), q66(n_materials)

!     outputs:
!     qbar11, qbar22, qbar12, qbar16, qbar26, qbar66
!     rho

	ct = cos(thp)
	st = sin(thp)

	c2t = ct*ct
	c3t = c2t*ct
	c4t = c3t*ct
	s2t = st*st
	s3t = s2t*st
	s4t = s3t*st
	s2thsq = 4.*s2t*c2t

      k11 = q11(mat)
      k22 = q22(mat)
      k12 = q12(mat)
      k66 = q66(mat)
      kmm = k11 -k12 -2.*k66
      kmp = k12 -k22 +2.*k66

      qbar11 = k11*c4t + 0.5*(k12+2.*k66)*s2thsq + k22*s4t
      qbar22 = k11*s4t + 0.5*(k12+2.*k66)*s2thsq + k22*c4t
      qbar12 = 0.25*(k11+k22-4.*k66)*s2thsq + k12*(s4t+c4t)
      qbar16 = kmm*st*c3t + kmp*s3t*ct
      qbar26 = kmm*s3t*ct + kmp*st*c3t
      qbar66 = 0.25*(kmm+k22-k12)*s2thsq  + k66*(s4t+c4t)

      rho_m = density(mat)

	return
	end

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	subroutine q_tildas

	USE QBars

!     inputs:
!     qbar11, qbar22, qbar12, qbar16, qbar26, qbar66

!     outputs:
!     qtil(2,2) matrix

      qtil(1,1) = qbar11 - qbar12*qbar12/qbar22
	if(qtil(1,1) .lt. 0.) then
	  write(*,*) '**ERROR: check material no ', mat, 
     *             ' properties; these are not physically realizable.'
	  stop 5151
	endif
      qtil(1,2) = qbar16 - qbar12*qbar26/qbar22
      qtil(2,2) = qbar66 - qbar26*qbar26/qbar22

	return
	end

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
